c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine read_atoms
#include "paradyn.h"
#include "mpif.h"
      integer istatus(mpi_status_size)      

      if (node.eq.0) write (6,*) 'Reading Atoms ...'
      
      if (ninterp.eq.0) call error('No interpolation done')
      nvelocities = 0

c find smallest lattice constant listed in potential file(s)
c except for Hydrogen (ielement = 1)
c used for allocating memory in setup_parallel
c this is not done for "create atoms" since latconst is user specified

      latconst = alat(1)
      do i = 2,npots
        if (alat(i).lt.latconst.and.ielement(i).ne.1)
     $       latconst = alat(i)
      enddo

c open atoms file - exit if there is a problem

      if (node.eq.0) then
        iflag = -1
        open (unit=9,file=atomfile,status='old',err=10)
        iflag = 0
      endif
      
 10   call mpi_bcast(iflag,1,mpi_integer,0,mpi_comm_world,ierror)
      if (iflag.lt.0) call error('Atom file cannot be opened')
      
c read time, box bounds, and # of atoms
      
      if (node.eq.0) then
        
        read (9,*)
        read (9,*) natoms
        read (9,*)
        read (9,*) perlb(1),perub(1)
        read (9,*) perlb(2),perub(2)
        read (9,*) perlb(3),perub(3)
        read (9,*)
        read (9,*) time_stamp
        read (9,*)
        
      endif
      
      call mpi_bcast(natoms,1,mpi_integer,0,mpi_comm_world,ierror)
      if (natoms.eq.0) call error('No atoms')
      
      call mpi_bcast(time_stamp,1,mpi_double_precision,0,
     $     mpi_comm_world,ierror)
      call mpi_bcast(perlb,3,mpi_double_precision,0,
     $     mpi_comm_world,ierror)
      call mpi_bcast(perub,3,mpi_double_precision,0,
     $     mpi_comm_world,ierror)
      
      call setup_parallel

c read atoms and send to appropriate processors
c ncount = # of atoms on processor iproc
c  (using same formula as in setup_parallel)
      
      if (node.eq.0) then

        do iproc = 0,nprocs-1
          ncount = nint(float(iproc+1)/nprocs*natoms) -
     $         nint(float(iproc)/nprocs*natoms)
          do i = 1,ncount
            read (9,*) tagtmp(i),itypetmp(i),
     $           xtmp(1,i),xtmp(2,i),xtmp(3,i)
          enddo

          if (iproc.eq.0) then
            do i = 1,ncount
              tag(i) = tagtmp(i)
              itype(i) = itypetmp(i)
              x(1,i) = xtmp(1,i)
              x(2,i) = xtmp(2,i)
              x(3,i) = xtmp(3,i)
            enddo
          else
            call mpi_send(tagtmp,ncount,mpi_integer,iproc,0,
     $           mpi_comm_world,ierror)
            call mpi_send(itypetmp,ncount,mpi_integer,iproc,0,
     $           mpi_comm_world,ierror)
            call mpi_send(xtmp,3*ncount,mpi_double_precision,
     $           iproc,0,mpi_comm_world,ierror)
          endif

        enddo
        close (9)

      else

        call mpi_recv(tag,maxlocal,mpi_integer,0,0,
     $       mpi_comm_world,istatus,ierror)
        call mpi_recv(itype,maxlocal,mpi_integer,0,0,
     $       mpi_comm_world,istatus,ierror)
        call mpi_recv(x,3*maxlocal,
     $       mpi_double_precision,0,0,mpi_comm_world,istatus,ierror)

      endif

      call mpi_barrier(mpi_comm_world,ierror)
      
c update everone's row and column atom tags and types
c use jneigh and nlarge_lst as temporary storage for counts and displacements

      call mpi_comm_size(row_comm,numcol,ierror)

      call mpi_allgather(nlocal,1,mpi_integer,jneigh,1,mpi_integer,
     $     row_comm,ierror)
      nlarge_lst(1) = 0
      do i = 2,numcol
        nlarge_lst(i) = nlarge_lst(i-1) + jneigh(i-1)
      enddo
      
      call mpi_allgatherv(tag,nlocal,mpi_integer,
     $     tagrow,jneigh,nlarge_lst,mpi_integer,row_comm,ierror)
      call mpi_allgatherv(itype,nlocal,mpi_integer,
     $     ityperow,jneigh,nlarge_lst,mpi_integer,row_comm,ierror)

      call mpi_comm_size(col_comm,numrow,ierror)

      call mpi_allgather(nlocal,1,mpi_integer,jneigh,1,mpi_integer,
     $     col_comm,ierror)
      nlarge_lst(1) = 0
      do i = 2,numrow
        nlarge_lst(i) = nlarge_lst(i-1) + jneigh(i-1)
      enddo

      call mpi_allgatherv(tag,nlocal,mpi_integer,
     $     tagcol,jneigh,nlarge_lst,mpi_integer,col_comm,ierror)
      call mpi_allgatherv(itype,nlocal,mpi_integer,
     $     itypecol,jneigh,nlarge_lst,mpi_integer,col_comm,ierror)

c set ntypes and check for consistency with potentials
      
      ntypes = 0
      do i = 1,nlocal
        ntypes = max(ntypes,itype(i))
      enddo
      call mpi_allreduce(ntypes,nmax,1,mpi_integer,mpi_max,
     $     mpi_comm_world,ierror)
      ntypes = nmax
      
      if (ntypes.gt.npots) then
        call error('More atom types than potentials')
      else if (ntypes.lt.npots) then
        if (node.eq.0)
     $       write (6,*) 'WARNING: Less atom types than potentials'
        if (logflag.ne.0.and.node.eq.0)
     $       write (1,*) 'WARNING: Less atom types than potentials'
      endif
      
      call chkper
      
c clear all controls and constraints on new atoms
      
      do i = 1,nlocal
        tmptag(i) = 0
        fixtag(i) = 0
      enddo
      
c zero out velocities

      do i = 1,nlocal
        v(1,i) = 0.0
        v(2,i) = 0.0
        v(3,i) = 0.0
      enddo

c message to screen
      
      if (logflag.ne.0.and.node.eq.0)
     $     write (1,*) 'Atoms Read =',natoms,' Types = ',ntypes
      if (node.eq.0) write (6,*) 'Atoms Read =',natoms,
     $     ' Types = ',ntypes
      
      return
      end
